@@ -9,7 +9,7 @@ use Sys::Hostname 'hostname';
 
 has 'mysql';
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 sub dequeue {
   my ($self, $id, $wait, $options) = @_;
@@ -38,9 +38,9 @@ sub enqueue {
 
   my $seconds = $db->dbh->quote($options->{delay} // 0);
   $db->query(
-    "insert into minion_jobs (`args`, `delayed`, `priority`, `queue`, `task`)
-     values (?, (DATE_ADD(NOW(), INTERVAL $seconds SECOND)), ?, ?, ?)",
-     encode_json($args), $options->{priority} // 0, 
+    "insert into minion_jobs (`args`, `attempts`, `delayed`, `priority`, `queue`, `task`)
+     values (?, ?, (DATE_ADD(NOW(), INTERVAL $seconds SECOND)), ?, ?, ?)",
+     encode_json($args), $options->{attempts} // 1, $options->{priority} // 0, 
     $options->{queue} // 'default', $task
   );
   my $job_id = $db->dbh->{mysql_insertid};
@@ -55,7 +55,7 @@ sub finish_job { shift->_update(0, @_) }
 
 sub job_info {
   my $hash = shift->mysql->db->query(
-    'select id, `args`, UNIX_TIMESTAMP(`created`) as `created`,
+    'select id, `args`, `attempts`, UNIX_TIMESTAMP(`created`) as `created`,
        UNIX_TIMESTAMP(`delayed`) as `delayed`,
        UNIX_TIMESTAMP(`finished`) as `finished`, `priority`, `queue`, `result`,
        UNIX_TIMESTAMP(`retried`) as `retried`, COALESCE(`retries`, 0) as retries,
@@ -142,14 +142,12 @@ sub repair {
   );
 
   # Abandoned jobs
-  $db->query(
-    "update minion_jobs as j
-     set finished = now(), result = ?,
-       state = 'failed'
+  my $fail = $db->query(
+    "select id, retries from minion_jobs as j
      where state = 'active'
-       and not exists(select 1 from minion_workers where id = j.worker)",
-   encode_json('Worker went away')
-  );
+       and not exists(select 1 from minion_workers where id = j.worker)"
+  )->hashes;
+  $fail->each(sub { $self->fail_job(@$_{qw(id retries)}, 'Worker went away') });
 
   # Old jobs
   $db->query(
@@ -174,11 +172,11 @@ sub retry_job {
 
   return !!$self->mysql->db->query(
     "update `minion_jobs`
-     set `finished` = 0, priority = coalesce(?, priority), 
-      `queue` = coalesce(?, queue), `result` = null, `retried` = now(),
-       `retries` = retries + 1, `started` = 0, `state` = 'inactive',
-       `worker` = null, `delayed` = (DATE_ADD(NOW(), INTERVAL $seconds SECOND))
-     where `id` = ? and retries = ? and `state` in ('failed', 'finished')",
+     set `priority` = coalesce(?, priority), `queue` = coalesce(?, queue),
+       `retried` = now(), `retries` = retries + 1, `state` = 'inactive',
+       `delayed` = (DATE_ADD(NOW(), INTERVAL $seconds SECOND))
+     where `id` = ? and retries = ? 
+       and `state` in ('failed', 'finished', 'inactive')",
      @$options{qw(priority queue)}, $id, $retries
   )->{affected_rows};
 }
@@ -203,12 +201,12 @@ sub stats {
   }
 
   return {
+    active_jobs => $states->{active} || 0,
     active_workers   => $active,
-    inactive_workers => $all - $active,
-    active_jobs      => $states->{active} || 0,
-    inactive_jobs    => $states->{inactive} || 0,
     failed_jobs      => $states->{failed} || 0,
     finished_jobs    => $states->{finished} || 0,
+    inactive_jobs    => $states->{inactive} || 0,
+    inactive_workers => $all - $active
   };
 }
 
@@ -243,6 +241,8 @@ sub _try {
 
   my $tasks = [keys %{$self->minion->tasks}];
 
+  return  unless @$tasks;  # Alexander Nalobin
+
   my $qq = join(", ", map({ "?" } @{ $options->{queues} // ['default'] }));
   my $qt = join(", ", map({ "?" } @{ $tasks }));
 
@@ -269,16 +269,35 @@ sub _try {
   $job;
 }
 
-sub _update {
+sub _update { ## Can this be made better?
   my ($self, $fail, $id, $retries, $result) = @_;
 
-  return !!$self->mysql->db->query(
+  my $db = $self->mysql->db;
+
+  my $tx = $db->begin;
+  my $row = $db->query(
+    "select attempts, id from minion_jobs 
+     where id = ? and retries = ? and state = 'active'
+     for update",
+     $id, $retries
+  )->array;
+
+  return undef if !defined $row;
+
+  $db->query(
     "update minion_jobs
      set finished = now(), result = ?, state = ?
-     where id = ? and retries = ? and state = 'active'",
-     encode_json($result), $fail ? 'failed' : 'finished', $id,
-    $retries
-  )->{affected_rows};
+     where id = ?",
+     encode_json($result), $fail ? 'failed' : 'finished',
+     $id
+  );
+  $tx->commit;
+  undef($tx);
+
+  return 1 if !$fail || (my $attempts = $row->[0]) == 1;
+  return 1 if $retries >= ($attempts - 1);
+  my $delay = $self->minion->backoff->($retries);
+  return $self->retry_job($id, $retries, {delay => $delay});
 }
 
 1;
@@ -298,15 +317,15 @@ Minion::Backend::mysql - MySQL backend
   # Slow task
   app->minion->add_task(poke_mojo => sub {
     my $job = shift;
-    $job->app->ua->get('mojolicio.us');
-    $job->app->log->debug('We have poked mojolicio.us for a visitor');
+    $job->app->ua->get('mojolicious.org');
+    $job->app->log->debug('We have poked mojolicious.org for a visitor');
   });
   
   # Perform job in a background worker process
   get '/' => sub {
     my $c = shift;
     $c->minion->enqueue('poke_mojo');
-    $c->render(text => 'We will poke mojolicio.us for you soon.');
+    $c->render(text => 'We will poke mojolicious.org for you soon.');
   };
 
   app->start;
@@ -315,8 +334,7 @@ Minion::Backend::mysql - MySQL backend
 
 L<Minion::Backend::mysql> is a backend for L<Minion> based on L<Mojo::mysql>. All
 necessary tables will be created automatically with a set of migrations named
-C<minion>. Note that this backend uses many bleeding edge features, so only the
-latest, stable version of PostgreSQL is fully supported.
+C<minion>. This backend has been tested on v5.5 of MySQL.
 
 =head1 ATTRIBUTES
 
@@ -438,7 +456,7 @@ Transition from C<active> to C<finished> state.
 
   my $job_info = $backend->job_info($job_id);
 
-Get information about a job or return C<undef> if job does not exist.
+Get information about a job, or return C<undef> if job does not exist.
 
   # Check job state
   my $state = $backend->job_info($job_id)->{state};
@@ -456,6 +474,12 @@ These fields are currently available:
 
 Job arguments.
 
+=item attempts
+
+  attempts => 25
+
+Number of times performing this job will be attempted, defaults to C<1>.
+
 =item created
 
   created => 784111777
@@ -563,7 +587,7 @@ Returns the same information as L</"worker_info"> but in batches.
 
 =head2 new
 
-  my $backend = Minion::Backend::mysql->new('postgresql://postgres@/test');
+  my $backend = Minion::Backend::mysql->new('mysql://mysql@/test');
 
 Construct a new L<Minion::Backend::mysql> object.
 
@@ -682,7 +706,7 @@ Time worker was started.
 
 =head1 SEE ALSO
 
-L<Minion>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
+L<Minion>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
 
 =cut
 
@@ -723,3 +747,6 @@ create index minion_jobs_state_idx on minion_jobs (state);
 
 -- 3 up
 alter table minion_jobs add queue varchar(128) not null default 'default';
+
+-- 4 up
+alter table minion_jobs add attempts int not null default 1;
